"""system lib init for pwclip"""
from system.clips import clips, copy, paste

from system.user import userfind, whoami

from system.path import filetime, setfiletime, filerotate

from system.xlib import xinput, xnotify, xyesno, xmsgok, xgetpass

from system.which import which



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""net.dns module"""

try:
	from os import uname
except ImportError:
	from os import environ
	uname = ['', environ['COMPUTERNAME']]

from re import search

from socket import getfqdn, gethostbyaddr, gaierror, herror

def isip(pattern):
	"""return true if input is possibly an ip-address"""
	# return True if "pattern" is RFC conform IP otherwise False
	iplike = r'^(?!0+\.0+\.0+\.0+|255\.255\.255\.255)' \
		r'(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)' \
		r'\.(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)$'
	if search(iplike, pattern):
		return True
	return False

def fqdn(name):
	"""get the fully qualified domain name"""
	__fqdn = getfqdn(name) if name else uname()[1]
	if __fqdn:
		return __fqdn
	return name

def askdns(host):
	"""ask dns for ip or name and return answer if ther is one"""
	try:
		dnsinfo = gethostbyaddr(host)
	except (gaierror, herror, TypeError):
		return host
	if isip(host):
		return dnsinfo[0]
	if len(dnsinfo[2]) == 1:
		return dnsinfo[2][0]
	return dnsinfo[2]

def raflookup(host):
	"""reverse and forward lookup function"""
	lookup, reverse = '', ''
	if host:
		lookup = askdns(host)
		if lookup:
			reverse = askdns(lookup)
	return lookup, reverse



"""x yes no question asking function"""
# -*- encoding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
#
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.

try:
	from tkinter import Button, Frame, Label, Tk
except ImportError:
	from Tkinter import Button, Frame, Label, Tk

class YesNo(Frame):
	"""password clipping class for tkinter.Frame"""
	yesno = False
	def __init__(self, master, message):
		self.message = message
		Frame.__init__(self, master)
		self.pack()
		self.inputwindow()
	def _enterexit(self, _=None):
		"""exit by saving challenge-response for input"""
		self.yesno = True
		self.quit()
	def _exit(self, _=None):
		"""just exit (for ESC mainly)"""
		self.quit()
	def inputwindow(self):
		"""password input window creator"""
		self.lbl = Label(self, text=message)
		self.lbl.pack()
		self.bind("<Return>", self._enterexit)
		self.bind("<Escape>", self._exit)
		self.pack()
		self.focus_set()
		self.ok = Button(self)
		self.ok["text"] = "ok"
		self.ok["command"] = self._enterexit
		self.ok.pack(side="left")
		self.cl = Button(self)
		self.cl["text"] = "cancel"
		self.cl["command"] = self._exit
		self.cl.pack(side="right")

def xyesno():
	"""gui representing function"""
	# instanciate Tk and create window
	root = Tk()
	xyn = YesNo(root, message="do you accept that?")
	xyn.lift()
	xyn.mainloop()
	root.destroy()
	return xyn.yesno



"""randomisation functions"""
from re import search
from os import urandom

def random(limit=10, regex=r'[\w -~]'):
	"""random character by limit"""
	retstr = ''
	while True:
		try:
			out = urandom(1).decode()
		except UnicodeDecodeError:
			continue
		try:
			if search(regex, out).group(0):
				retstr = '%s%s'%(retstr, out.strip())
		except AttributeError:
			continue
		if len(retstr) >= int(limit):
			break
	return retstr

def biggerrand(num):
	"""greater random number"""
	while True:
		try:
			g = int(random(int(len(str(num))+1), regex=r'[0-9]*'))
		except ValueError:
			continue
		if g > int(num):
			break
	return g

def lowerrand(num):
	"""lower random number"""
	while True:
		try:
			g = int(random(int(len(str(num))), regex=r'[0-9]*'))
		except ValueError:
			continue
		if g > 1 and g < int(num):
			break
	return g

def randin(top, low=0):
	"""number in between low and top"""
	if low:
		low, top = top, low
	while True:
		try:
			g = int(random(int(len(str(top))), regex=r'[0-9]*'))
		except ValueError:
			continue
		if g > int(low) and g < int(top):
			break
	return g



#!/usr/bin/env python3
"""pwclip packaging information"""
from os import getcwd, path

name = distname = modname = 'pwclip'
numversion = (1, 2, 10)
version = '.'.join([str(num) for num in numversion])
provides = ['pwcli', 'pwclip', 'ykclip']
install_requires = [
    'argcomplete', 'netaddr', 'paramiko', 'psutil', 'pyusb',
    'python-gnupg', 'python-yubico', 'PyYAML', 'wget']
lic = 'GPL'
description = "gui to temporarily save passwords to system-clipboard"
mailinglist = ""
author = 'Leon Pelzer'
author_email = 'mail@leonpelzer.de'
download_url = 'https://pypi.python.org/pypi/pwclip/%s#downloads'%version
classifiers = ['Environment :: Console',
               'Environment :: MacOS X',
               'Environment :: Win32 (MS Windows)',
               'Environment :: X11 Applications',
               'Intended Audience :: Developers',
               'Intended Audience :: End Users/Desktop',
               'Intended Audience :: System Administrators',
               'Intended Audience :: Information Technology',
               'License :: OSI Approved :: GNU General Public License (GPL)',
               'Operating System :: OS Independent',
               'Programming Language :: Python :: 3',
               'Topic :: Security',
               'Topic :: Utilities',
               'Topic :: Desktop Environment',
               'Topic :: System :: Systems Administration']
include_package_data = True
try:
	with open(path.join(getcwd(), 'CHANGELOG.rst'), 'r') as cfh:
		changelog = '\n\n\n'.join(cfh.read().split('\n\n\n')[:4])
	with open(path.join(getcwd(), 'README.rst'), 'r') as rfh:
		readme = rfh.read().format(ChangeLog=changelog)
	with open(path.join(getcwd(), 'README'), 'w+') as rfh:
		rfh.write(readme)
except OSError:
	readme = ''
long_desc = (readme)
entry_points = {
    'gui_scripts': ['pwclip = pwclip.__init__:pwclip',
                    'ykclip = pwclip.__init__:ykclip'],
    'console_scripts': ['pwcli = pwclip.__init__:pwcli']}



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
#
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
"""
clips - clipboard for various systems
"""
from os import name as osname

from platform import system

from time import sleep, time

from subprocess import Popen, PIPE, DEVNULL

def winclips():
	"""windows clipboards - the ugliest thing i've ever seen"""
	from ctypes import \
        windll, memmove, c_size_t, \
        sizeof, c_wchar_p, get_errno, c_wchar
	from ctypes.wintypes import \
        INT, HWND, DWORD, LPCSTR, HGLOBAL, \
        LPVOID, HINSTANCE, HMENU, BOOL, UINT, HANDLE
	from contextlib import contextmanager
	GMEM_MOVEABLE = 0x0002
	CF_UNICODETEXT = 13
	class CheckedCall(object):
		"""fancy windows callchecker"""
		def __init__(self, f):
			self.f = f
			super(CheckedCall, self).__setattr__("f", f)
		def __call__(self, *args):
			ret = self.f(*args)
			if not ret and get_errno():
				raise RuntimeError("Error calling %s"%self.f.__name__)
			return ret
		def __setattr__(self, key, value):
			setattr(self.f, key, value)
	mkwin = CheckedCall(windll.user32.CreateWindowExA)
	setattr('mkwin', 'argtypes', [
        DWORD, LPCSTR, LPCSTR, DWORD, INT, INT,
        INT, INT, HWND, HMENU, HINSTANCE, LPVOID])
	setattr('mkwin', 'restype', HWND)
	delwin = CheckedCall(windll.user32.DestroyWindow)
	setattr('delwin', 'argtypes', [HWND])
	setattr('delwin', 'restype', BOOL)
	clip = windll.user32.OpenClipboard
	setattr('clip', 'argtypes', [HWND])
	setattr('clip', 'restype', BOOL)
	clsclip = CheckedCall(windll.user32.CloseClipboard)
	setattr('clsclip', 'argtypes', [])
	setattr('clsclip', 'restype', BOOL)
	delclip = CheckedCall(windll.user32.EmptyClipboard)
	setattr('delclip', 'argtypes', [])
	setattr('delclip', 'restype', BOOL)
	getclip = CheckedCall(windll.user32.GetClipboardData)
	setattr('getclip', 'argtypes', [UINT])
	setattr('getclip', 'restype', HANDLE)
	setclip = CheckedCall(windll.user32.SetClipboardData)
	setattr('setclip', 'argtypes', [UINT, HANDLE])
	setattr('setclip', 'restype', HANDLE)
	allock = CheckedCall(windll.kernel32.GlobalAlloc)
	setattr('allock', 'argtypes', [UINT, c_size_t])
	setattr('setclip', 'restype', HGLOBAL)
	dolock = CheckedCall(windll.kernel32.GlobalLock)
	setattr('dolock', 'argtypes', [HGLOBAL])
	setattr('doclip', 'restype', LPVOID)
	unlock = CheckedCall(windll.kernel32.GlobalUnlock)
	setattr('unlock', 'argtypes', [HGLOBAL])
	setattr('unclip', 'restype', BOOL)
	@contextmanager
	def window():
		"""context that provides a valid window (hwnd)"""
		hwnd = mkwin(0, b"STATIC", None, 0, 0, 0, 0, 0, None, None, None, None)
		try:
			yield hwnd
		finally:
			delwin(hwnd)
	@contextmanager
	def clipboard(hwnd):
		"""windows clipboard context manager"""
		t = time() + 0.5
		success = False
		while time() < t:
			success = clip(hwnd)
			if success:
				break
			sleep(0.01)
		if not success:
			raise Exception("could not open clipboard")
		try:
			yield
		finally:
			clsclip()
	def _copy(text, mode=None):
		"""windows copy function"""
		if mode == 'b':
			return
		with window() as hwnd:
			with clipboard(hwnd):
				delclip()
				if text:
					count = len(text) + 1
					handle = allock(GMEM_MOVEABLE, count*sizeof(c_wchar))
					locked_handle = dolock(handle)
					memmove(
                        c_wchar_p(locked_handle),
                        c_wchar_p(text), count*sizeof(c_wchar))
					unlock(handle)
					setclip(CF_UNICODETEXT, handle)
	def _paste(_=None):
		"""windows paste function"""
		with clipboard(None):
			handle = getclip(CF_UNICODETEXT)
			if not handle:
				return "", ""
		return c_wchar_p(handle).value, c_wchar_p(handle).value
	return _copy, _paste

def osxclips():
	""""OSX clipboards"""
	def _copy(text, mode=None):
		"""osx copy function"""
		text = text if text else ''
		if mode != 'b':
			with Popen(['pbcopy'], stdin=PIPE, close_fds=True) as prc:
				prc.communicate(input=str(text).encode('utf-8'))
		return False
	def _paste(_=None):
		"""osx paste function"""
		with Popen(['pbpaste'], stdout=PIPE, close_fds=True) as prc:
			out, _ = prc.communicate()
			return out.decode('utf-8'), None
		return False
	return _copy, _paste

def linclips():
	"""linux clipboards"""
	xsel = ['xsel']
	def _copy(text, mode='p'):
		"""linux copy function"""
		_xsel = xsel + ['-i']
		text = text if text else ''
		for m in mode:
			with Popen(
				  _xsel + ['-%s'%m], stdin=PIPE, stderr=DEVNULL) as prc:
				prc.communicate(input=str(text).encode('utf-8'))
				prc.terminate()
	def _paste(mode='p'):
		"""linux paste function"""
		_xsel = xsel + ['-o']
		if mode == 'p':
			out, _ = Popen(
				_xsel + ['-p'], stdout=PIPE, stderr=DEVNULL).communicate()
			ret = out.decode()
		elif mode == 'b':
			out, _ = Popen(
				_xsel + ['-b'], stdout=PIPE, stderr=DEVNULL).communicate()
			ret = out.decode()
		else:
			pout, _ = Popen(
				_xsel + ['-p'], stdout=PIPE, stderr=DEVNULL).communicate()
			bout, _ = Popen(
				_xsel + ['-b'], stdout=PIPE, stderr=DEVNULL).communicate()
			ret = pout.decode(), bout.decode()
		return ret
	return _copy, _paste

def clips():
	"""return `copy`, `paste` as system independent functions"""
	if osname == 'nt' or system() == 'Windows':
		return winclips()
	elif osname == 'mac' or system() == 'Darwin':
		return osxclips()
	return linclips()

copy, paste = clips()
